package ru.dbotthepony.mc.otm.android

import it.unimi.dsi.fastutil.objects.ObjectArraySet
import net.minecraft.data.CachedOutput
import net.minecraft.data.DataGenerator
import net.minecraft.data.DataProvider
import net.minecraft.data.PackOutput
import net.minecraft.resources.ResourceLocation
import java.util.Collections
import java.util.LinkedList
import java.util.concurrent.CompletableFuture
import java.util.function.Consumer

@Suppress("unused")
open class AndroidResearchDataProvider(protected val dataGenerator: DataGenerator, val modid: String) : DataProvider {
	protected val callbacks = LinkedList<(Consumer<AndroidResearchType>) -> Unit>()

	private val generated = LinkedList<AndroidResearchType>()
	val generatedView: List<AndroidResearchType> = Collections.unmodifiableList(generated)

	/**
	 * override this
	 */
	protected open fun addEverything(serializer: Consumer<AndroidResearchType>) {
		for (callback in callbacks) {
			callback.invoke(serializer)
		}
	}

	/**
	 * or call this
	 */
	fun exec(callback: (Consumer<AndroidResearchType>) -> Unit): AndroidResearchDataProvider {
		callbacks.add(callback)
		return this
	}

	final override fun run(output: CachedOutput): CompletableFuture<*> {
		AndroidResearchManager.fireRegistrationEvent()

		val set = ObjectArraySet<ResourceLocation>()
		val added = LinkedList<AndroidResearchType>()
		val futures = ArrayList<CompletableFuture<*>>()

		val path = dataGenerator.packOutput.getOutputFolder(PackOutput.Target.DATA_PACK).resolve(modid).resolve(AndroidResearchManager.DIRECTORY)

		addEverything {
			if (set.add(it.id)) {
				futures.add(DataProvider.saveStable(output, it.toJson(), path.resolve("${it.id}.json")))
				AndroidResearchManager.put(it)
				added.add(it)
			} else {
				throw IllegalStateException("Duplicate android research with ID ${it.id}")
			}
		}

		for (value in added) {
			value.validate()
			generated.add(value)
		}

		return CompletableFuture.allOf(*futures.toTypedArray())
	}

	override fun getName(): String {
		return "Android Research"
	}
}
